package com.streambase.sb.contrib.liveview.logparse;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.PrintStream;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Read from the LV server logs and write a csv file that contains the queries.
 *
 */
public class LVLogQueriesToCSV {

	// 2013-07-16 12:47:16.718 INFO  LVInfoQuery - { 'LiveViewStatistics'::Original Query }"{""CQSConnectionID"":""1EA3BC4B9CB2C2A53FEAA5CB7533A947"",""CQSQueryID"":""e39faf47-1bb7-46d5-adb1-bac585d18abb:1"",""_PredicateExpr"":""TableName=='Items' && RecordCount > 100"",""_Command"":4,""_MaxRecordsLimit"":2147483647}"
	private final static Pattern LINE_PAT_1_4_5 = 
			Pattern.compile("(\\d+-\\d\\d-\\d\\d [0-9:.]+).* LVInfoQuery .*[{] '([a-zA-Z0-9_]+)'::Original Query [}](.*)$");
	
	private BufferedReader in;
	private PrintStream out;

	public LVLogQueriesToCSV(String fileName, PrintStream out) throws IOException {
		in = new BufferedReader(new FileReader(fileName));
		this.out = out;
	}
	
	public void processLog() throws IOException {
		String line;

		try {
			writeHeader();

			while((line = in.readLine()) != null) {
				parseLine(line);
			}
		} finally {
			closeAll();
		}		
	}

	private void parseLine(String line) throws IOException {
		Matcher m = LINE_PAT_1_4_5.matcher(line);

		if(m.matches()) {
			String time = m.group(1);
			String table = m.group(2);
			String jsonString = m.group(3);

			out.print(time);
			out.print(",");
			out.print(table);
			out.print(",");
			out.print(jsonString);

			out.print("\n");
		}
	}


	private void writeHeader() throws IOException {
		out.println("time,table,JSONString");
	}
	
	private void closeAll() {
		try {
			out.close();
			in.close();
			out = null;
			in = null;
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args)  {
		if(args.length < 1) {
			help();
		}
		
		PrintStream out = System.out;
		
		if(args.length > 1) {
			if (!"-o".equals(args[1]) || (args.length < 3)) {
				help();
			}
			try {
				out = new PrintStream(args[2]);
			} catch (FileNotFoundException e) {
				out.println("Can't find file: " + args[2]);
				help();
			}
		}
		
		try {
			LVLogQueriesToCSV lvlog = new LVLogQueriesToCSV(args[0], out);
			lvlog.processLog();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	private static void help() {
		System.err.println("Usage: java " + LVLogQueriesToCSV.class + " logfile [ -o OutputFilePath ]");
		System.exit(-1);
	}
}
